﻿using System;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Drawing;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using System.Drawing.Printing;

namespace PrinterGeoCanvasExtensionExtensionSamples
{
    public partial class Sample : Form
    {
        private MapEngine mapEngine;
        private Bitmap bitmap;

        public Sample()
        {
            InitializeComponent();
        }

        // This method is really standard.  It is used to draw the image on the screen.  
        // The real magic is in the btnToPdf_Click event.
        private void Sample_Load(object sender, EventArgs e)
        {
            bitmap = new Bitmap(map.Width, map.Height);

            mapEngine = new MapEngine();

            mapEngine.CurrentExtent = new RectangleShape(-135.47672664498, 53.1062529845545, -76.20327335502, 12.4645242175181);

            mapEngine.BackgroundFillBrush = new GeoSolidBrush(GeoColor.GeographicColors.ShallowOcean);
            EcwRasterLayer worldImageLayer = new EcwRasterLayer(@"..\..\SampleData\World.ecw");
            ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(@"..\..\SampleData\Countries02.shp");
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.OutlinePen.StartCap = DrawingLineCap.Round;
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.OutlinePen.EndCap = DrawingLineCap.Round;
            worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            mapEngine.StaticLayers.Add("WorldImageLayer", worldImageLayer);
            mapEngine.StaticLayers.Add("WorldLayer", worldLayer);

            DrawImage();
        }

        // Here we setup the PDF page and then create our PDFGeoCanvas.
        // We loop through all the layers to draw and then save & pop the 
        // PDF
        private void btnToPrinter_Click(object sender, EventArgs e)
        {
            PrinterGeoCanvas printerGeoCanvas = new PrinterGeoCanvas();

            // This allows you to control the area in which you want the
            // map to draw in.  Leaving this commented out uses the whole page
            //pdfGeoCanvas.DrawingArea = new Rectangle(200, 50, 400, 400);
            Collection<SimpleCandidate> labelsInLayers = new Collection<SimpleCandidate>();
            PrintDocument document = new PrintDocument();
            printerGeoCanvas.DrawingArea = new Rectangle(0, 0, 579 , 397 );
            printerGeoCanvas.BeginDrawing(document, mapEngine.CurrentExtent, GeographyUnit.DecimalDegree);
            foreach (Layer layer in mapEngine.StaticLayers)
            {
                layer.Open();
                layer.Draw(printerGeoCanvas, labelsInLayers);
                layer.Close();
                printerGeoCanvas.Flush();
            }

            printerGeoCanvas.EndDrawing();
        }



        private void btnPreview_Click(object sender, EventArgs e)
        {
            PrintPreviewDialog dialog = new PrintPreviewDialog();
            dialog.Document = new PrintDocument();

            Collection<SimpleCandidate> labelsInLayers = new Collection<SimpleCandidate>();
            PrintDocument document = dialog.Document;

            PrinterGeoCanvas printerGeoCanvas = new PrinterGeoCanvas();
            printerGeoCanvas.DrawingArea = new Rectangle(0, 0, 579 , 397);
            printerGeoCanvas.BeginDrawing(document, mapEngine.CurrentExtent, GeographyUnit.DecimalDegree);
            printerGeoCanvas.Clear(new GeoSolidBrush(GeoColor.SimpleColors.Transparent));
            foreach (Layer layer in mapEngine.StaticLayers)
            {
                layer.Open();
                layer.Draw(printerGeoCanvas, labelsInLayers);
                layer.Close();
                printerGeoCanvas.Flush();
            }

            ((ToolStrip)(dialog.Controls[1])).Items[0].Click += new EventHandler(CallPrinterEndDrawing); ;

            dialog.ShowDialog();
        }


        public void CallPrinterEndDrawing(object sender, EventArgs e)
        {
            //PrinterGeoCanvas printerGeoCanvas = new PrinterGeoCanvas();
            //printerGeoCanvas.DrawingArea = new Rectangle(0, 0, 579 * 4, 397 * 4);
            //printerGeoCanvas.BeginDrawing(document, mapEngine.CurrentExtent, GeographyUnit.DecimalDegree);
            //printerGeoCanvas.Clear(new GeoSolidBrush(GeoColor.SimpleColors.Transparent));
            //Collection<SimpleCandidate> labelsInLayers = new Collection<SimpleCandidate>();
            //foreach (Layer layer in mapEngine.StaticLayers)
            //{
            //    layer.Open();
            //    layer.Draw(printerGeoCanvas, labelsInLayers);
            //    layer.Close();
            //    printerGeoCanvas.Flush();
            //}

            btnToPrinter_Click(sender, e);
        }

        private void DrawImage()
        {
            mapEngine.OpenAllLayers();
            mapEngine.DrawStaticLayers(bitmap, GeographyUnit.DecimalDegree);
            mapEngine.CloseAllLayers();
            map.Image = bitmap;
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void ToolBar_ButtonClick(object sender, ToolBarButtonClickEventArgs e)
        {
            switch (e.Button.Tag.ToString())
            {
                case "Zoom In":
                    mapEngine.CurrentExtent.ScaleDown(30);
                    Refresh();
                    break;
                case "Zoom Out":
                    mapEngine.CurrentExtent.ScaleUp(30);
                    Refresh();
                    break;
                case "Pan Left":
                    mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Left, 30);
                    Refresh();
                    break;
                case "Pan Right":
                    mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Right, 30);
                    Refresh();
                    break;
                case "Pan Up":
                    mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Up, 30);
                    Refresh();
                    break;
                case "Pan Down":
                    mapEngine.CurrentExtent = ExtentHelper.Pan(mapEngine.CurrentExtent, PanDirection.Down, 30);
                    Refresh();
                    break;
                default:
                    break;
            }
            DrawImage();
        }
    }
}